# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SendMessage Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase, AttachFile
from MoinMoin.formatter import text_html
from MoinMoin.log import getLogger
from MoinMoin import config
from MoinMessage import GPG, MoinMessageError, Message, sendMessage
from MoinSupport import *
from MoinMoin.wikiutil import escape, MimeType, parseQueryString, taintfilename

from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import urllib

Dependencies = []

class SendMessage(ActionBase, ActionSupport):

    "An action that can send a message to another site."

    def get_form_html(self, buttons_html):

        "Present an interface for message sending."

        _ = self._
        request = self.request
        form = self.get_form()

        message = form.get("message", [""])[0]
        recipient = form.get("recipient", [""])[0]
        preview = form.get("preview", [""])[0]

        # Get a list of potential recipients.

        recipients = self.get_recipients()

        # Prepare the recipients list, selecting the specified recipients.

        recipients_list = []

        if recipients:
            recipients_list += self.get_option_list(recipient, recipients) or []

        recipients_list.sort()

        # Prepare any preview.

        request.formatter.setPage(self.page)
        preview_output = preview and formatText(message, request, request.formatter, inhibit_p=False) or ""

        # Fill in the fields and labels.

        d = {
            "buttons_html"          : buttons_html,
            "recipient_label"       : _("Recipient"),
            "recipients_list"       : "\n".join(recipients_list),
            "message_label"         : _("Message text"),
            "message_default"       : escape(message),
            "preview_label"         : _("Preview message"),
            "preview_output"        : preview_output,
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td class="label"><label>%(recipient_label)s</label></td>
        <td>
            <select name="recipient">
                %(recipients_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(message_label)s</label></td>
        <td>
            <textarea name="message" cols="60" rows="10">%(message_default)s</textarea>
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            <input name="preview" type="submit" value="%(preview_label)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="moinmessage-preview">
%(preview_output)s
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Attempt to send the message."

        _ = self._
        request = self.request
        form = self.get_form()

        text = form.get("message", [None])[0]
        recipient = form.get("recipient", [None])[0]

        if not text:
            return 0, _("A message must be given.")

        if not recipient:
            return 0, _("A recipient must be given.")

        homedir = self.get_homedir()
        if not homedir:
            return 0, _("MoinMessage has not been set up: a GPG homedir is not defined.")

        gpg = GPG(homedir)

        # Construct a message from the request.

        message = Message()

        container = MIMEMultipart("related")
        container["Update-Action"] = "store"

        # Add the message body and any attachments.

        fmt = OutgoingHTMLFormatter(request)
        fmt.setPage(request.page)
        body = formatText(text, request, fmt)

        container.attach(MIMEText(body, "html"))

        for pos, (pagename, filename) in enumerate(fmt.attachments):

            # Obtain the attachment path.

            filename = taintfilename(filename)
            path = AttachFile.getFilename(request, pagename, filename)

            # Obtain the attachment content.

            f = open(path, "rb")
            try:
                body = f.read()
            finally:
                f.close()

            # Determine the attachment type.

            mimetype = MimeType(filename=filename)

            # NOTE: Support a limited set of explicit part types for now.

            if mimetype.major == "image":
                part = MIMEImage(body, mimetype.minor, **mimetype.params)
            elif mimetype.major == "text":
                part = MIMEText(body, mimetype.minor, mimetype.charset, **mimetype.params)
            else:
                part = MIMEApplication(body, mimetype.minor, **mimetype.params)

            # Label the attachment and include it in the message.

            part["Content-ID"] = "attachment%d" % pos
            container.attach(part)

        message.add_update(container)

        # Get the sender details for signing messages.
        # This is not the same as the details for authenticating users in the
        # PostMessage action since the fingerprints refer to public keys.

        signing_users = self.get_signing_users()
        signer = signing_users and signing_users.get(request.user.name)

        # Get the recipient details.

        recipients = self.get_recipients()
        if not recipients:
            return 0, _("No recipients page is defined for MoinMessage.")

        recipient_details = recipients.get(recipient)
        if not recipient_details:
            return 0, _("The specified recipient is not present in the list of known contacts.")

        try:
            url, fingerprint = recipient_details.split()
        except ValueError:
            return 0, _("The recipient details were not in the correct format: url, fingerprint.")

        # Sign, encrypt and send the message.

        try:
            message = message.get_payload()
            if signer:
                message = gpg.signMessage(message, signer)

            message = gpg.encryptMessage(message, fingerprint)
            sendMessage(message, url)

        except MoinMessageError, exc:
            return 0, "%s: %s" % (_("The message could not be prepared and sent:"), exc)

        return 1, None

    def get_homedir(self):

        "Locate the GPG home directory."

        return getattr(self.request.cfg, "moinmessage_gpg_homedir")

    def get_recipients(self):
        return getWikiDict(
            getattr(self.request.cfg, "moinmessage_gpg_recipients_page", "MoinMessageRecipientsDict"),
            self.request)

    def get_signing_users(self):
        return getWikiDict(
            getattr(self.request.cfg, "moinmessage_gpg_signing_users_page", "MoinMessageSigningUserDict"),
            self.request)

# Special message formatters.

def unquoteWikinameURL(url, charset=config.charset):

    """
    The inverse of wikiutil.quoteWikinameURL, returning the page name referenced
    by the given 'url', with the page name assumed to be encoded using the given
    'charset' (or default charset if omitted).
    """

    return unicode(urllib.unquote(url), encoding=charset)

def getAttachmentFromURL(url, request):

    """
    Return a (page name, attachment filename) tuple for the attachment
    referenced by the given 'url', using the 'request' to interpret the
    structure of 'url'.

    If 'url' does not refer to an attachment on this wiki, None is returned.
    """

    script = request.getScriptname()
    if not url.startswith(script):
        return None

    path = url[len(script):].lstrip("/")
    try:
        qpagename, qs = path.split("?", 1)
    except ValueError:
        qpagename = path
        qs = None

    pagename = unquoteWikinameURL(qpagename)
    qs = qs and parseQueryString(qs) or {}
    return pagename, qs.get("target") or qs.get("drawing")

class OutgoingHTMLFormatter(text_html.Formatter):

    """
    Handle outgoing HTML content by identifying attachments and rewriting their
    locations. References to bundled attachments are done using RFC 2111:

    https://tools.ietf.org/html/rfc2111

    Messages employing references between parts are meant to comply with RFC
    2387:

    https://tools.ietf.org/html/rfc2387
    """

    def __init__(self, request, **kw):
        text_html.Formatter.__init__(self, request, **kw)
        self.attachments = []

    def add_attachment(self, location):
        details = getAttachmentFromURL(location, self.request)
        if details:
            pos = len(self.attachments)
            self.attachments.append(details)
            return "cid:attachment%d" % pos
        else:
            return None

    def image(self, src=None, **kw):
        src = src or kw.get("src")
        ref = src and self.add_attachment(src)
        return text_html.Formatter.image(self, ref or src, **kw)

    def transclusion(self, on, **kw):
        if on:
            data = kw.get("data")
            kw["data"] = data and self.add_attachment(data)
        return text_html.Formatter.transclusion(self, on, **kw)

# Action function.

def execute(pagename, request):
    SendMessage(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
